%
% Copyright (C) 2014 Luis de la Ossa, Jesús Martínez, Sara Sáez, Joaquín Jordán.
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%  
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%  
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
%
%

function [dataset] = generateDataSet(cellDataByImage)
% Generates a plain dataset with the data obtained from the images.

% cellDataByImage: Cell array which contains the information extracted from each image.
%                Each position, which corresponds to a image contains an array of structs (cellData), 
%                each structure corresponds to a cell, and contains these fields:
%                 
%        cellData.numPixels: Number of pixels of the cell.
%        cellData.numPixelsOverUpperThr:  Number of pixels over params.upperBackgroundThreshold.
%        cellData.propPixelsOverUpperThr:  Proportion of pixels over params.upperBackgroundThreshold.
%        cellData.numPixelsOverAggregationThr:  Number of pixels over params.aggregationThreshold.
%        cellData.propPixelsOverAggregationThr:  Proportion of pixels over params.aggregationThreshold.         
%        cellData.majorAxisLength:  Lenght of the major axis of the ellipse containing the object.
%        cellData.minorAxisLength:  Lenght of the minor axis of the ellipse containing the object.
%        cellData.cellAvgColor:  Average color of the object
%        cellData.imgNumPixelsOverUpperThr:  Number of pixels over params.upperBackgroundThreshold in the whole image.
%        cellData.imgPropPixelsOverUpperThr: Proportion  of pixels over params.upperBackgroundThreshold in the whole image.
%        cellData.imgNumPixelsOverAggreThr:  Number of pixels over params.aggregationThreshold in the whole image.
%        cellData.imgPropPixelsOverAggreThr: Proportion  of pixels over params.aggregationThreshold in the whole image.                    
%        cellData.imgAvgColor:  Average color of the image.
%        cellData.cellPOUTvsimgPOUT:  cellData.propPixelsOverUpperThr / cellData.imgPropPixelsOverUpperThr.
%        cellData.cellPOATvsimgPOAT:  cellData.propPixelsOverAggregationThr / cellData.imgPropPixelsOverAggreThr.
%        cellData.cellColorVSImgColor:  cellData.cellAvgColor / cellData.imgAvgColor.
%        cellData.Hist:  Histogram of the object subimage.     
%        cellData.Class: Class of the cell. ([ ]=Discarded, [2] = Diffuse Aggregation, [3] = Translocated Aggregation).   
                       
  
% dataset: Plain data. Contains two fields.

        % dataset.Class:  Class of the cell (numCelulas x 1).
                       % [ ]=Discarded, [2] = Diffuse Aggregation, [3] = Translocated Aggregation
                       
        % dataset.Input: Information related to each cell (numCelulas x 272).
%            1-->  Number of pixels of the cell.
%            2-->  Number of pixels over params.upperBackgroundThreshold.
%            3-->  Proportion of pixels over params.upperBackgroundThreshold.
%            4-->  Number of pixels over params.aggregationThreshold.
%            5-->  Proportion of pixels over params.aggregationThreshold.            
%            6-->  Lenght of the major axis of the ellipse containing the object.
%            7-->  Lenght of the minor axis of the ellipse containing the object.
%            8-->  Average color of the object
%            9-->  Number of pixels over params.upperBackgroundThreshold in the whole image.
%           10-->  Proportion  of pixels over params.upperBackgroundThreshold in the whole image.
%           11-->  Number of pixels over params.aggregationThreshold in the whole image.
%           12-->  Proportion  of pixels over params.aggregationThreshold in the whole image.                    
%           13-->  Average color of the image
%           14-->  cellData.propPixelsOverUpperThr / cellData.imgPropPixelsOverUpperThr.  
%           15-->  cellData.propPixelsOverAggregationThr / cellData.imgPropPixelsOverAggreThr.  
%           16-->  cellData.cellAvgColor / cellData.imgAvgColor.
%       17-272-->  Histogram of the object subimage.  

  
    auxDataArray = cat(2,cellDataByImage{:});
    numCells = numel(auxDataArray);

    dataset.Class = [auxDataArray.Class]';
    dataset.Input = zeros(numCells,272);

    dataset.Input(:,1) = [auxDataArray.numPixels]';
    dataset.Input(:,2) = [auxDataArray.numPixelsOverUpperThr]';
    dataset.Input(:,3) = [auxDataArray.propPixelsOverUpperThr]';
    dataset.Input(:,4) = [auxDataArray.numPixelsOverAggregationThr]';    
    dataset.Input(:,5) = [auxDataArray.propPixelsOverAggregationThr]';
    dataset.Input(:,6) = [auxDataArray.majorAxisLength]';
    dataset.Input(:,7) = [auxDataArray.minorAxisLength]';
    dataset.Input(:,8) = [auxDataArray.cellAvgColor]';
    dataset.Input(:,9) = [auxDataArray.imgNumPixelsOverUpperThr]';
    dataset.Input(:,10) = [auxDataArray.imgPropPixelsOverUpperThr]';
    dataset.Input(:,11) = [auxDataArray.imgNumPixelsOverAggreThr]';
    dataset.Input(:,12) = [auxDataArray.imgPropPixelsOverAggreThr]';
    dataset.Input(:,13) = [auxDataArray.imgAvgColor]';    
    dataset.Input(:,14) = [auxDataArray.cellPOUTvsimgPOUT]';      
    dataset.Input(:,15) = [auxDataArray.cellPOATvsimgPOAT]';     
    dataset.Input(:,16) = [auxDataArray.cellColorVSImgColor]';     
    dataset.Input(:,17:272) = [auxDataArray.Hist]';
end

