%
%
% Copyright (C) 2014 Luis de la Ossa, Jesús Martínez, Sara Sáez, Joaquín Jordán.
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%  
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%  
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
%

function [cellLocationsByImage cellDataByImage imageNames] = getInfoFromImages(folder, imageNameFilter, params)
% Extracts the information from a set of images.

% folder: Path to the folder which contain the images.

% imageNameFilter: Ej. '.*jpg';

% params: Struct which contains the values for the parameters of the algorithm.
%
%      params.lowerBackgroundThreshold: Pixels above this value are considered objects of interest.
%      params.upperBackgroundThreshold: Used to segment accumulations of cells.
%      params.aggregationThreshold: Threshold used to detect aggregations of proteins.
%      params.minCellSize: Minimum cell size.
%      params.minCumSize: Minimum size of an accumulation of cells.
%      params.smallCellMaxSize: Maximum size of a cell considered as small.
%      params.askForClass: Whether the program must ask for the kind of cell.

% cellLocationsByImage: Cell array which contains the information the information extracted from 
%                       each image in relation with the position of the cells. Each position 
%                       (which corresponds to an image) contains an array of structs (cellLocations)
%                       which correspond to a cell. Each struct has two fields:
%
%             cellLocations.BoundingBox: Coordinates of the rectangle containing the object (4x1)
%             cellLocations.Centroid: Centroid of the object.   

% cellDataByImage: Cell array which contains the information extracted from each image.
%                Each position, which corresponds to a image contains an array of structs (cellData), 
%                each structure corresponds to a cell, and contains these fields:
%                 
%        cellData.numPixels: Number of pixels of the cell.
%        cellData.numPixelsOverUpperThr:  Number of pixels over params.upperBackgroundThreshold.
%        cellData.propPixelsOverUpperThr:  Proportion of pixels over params.upperBackgroundThreshold.
%        cellData.numPixelsOverAggregationThr:  Number of pixels over params.aggregationThreshold.
%        cellData.propPixelsOverAggregationThr:  Proportion of pixels over params.aggregationThreshold.         
%        cellData.majorAxisLength:  Lenght of the major axis of the ellipse containing the object.
%        cellData.minorAxisLength:  Lenght of the minor axis of the ellipse containing the object.
%        cellData.cellAvgColor:  Average color of the object
%        cellData.imgNumPixelsOverUpperThr:  Number of pixels over params.upperBackgroundThreshold in the whole image.
%        cellData.imgPropPixelsOverUpperThr: Proportion  of pixels over params.upperBackgroundThreshold in the whole image.
%        cellData.imgNumPixelsOverAggreThr:  Number of pixels over params.aggregationThreshold in the whole image.
%        cellData.imgPropPixelsOverAggreThr: Proportion  of pixels over params.aggregationThreshold in the whole image.                    
%        cellData.imgAvgColor:  Average color of the image.
%        cellData.cellPOUTvsimgPOUT:  cellData.propPixelsOverUpperThr / cellData.imgPropPixelsOverUpperThr.
%        cellData.cellPOATvsimgPOAT:  cellData.propPixelsOverAggregationThr / cellData.imgPropPixelsOverAggreThr.
%        cellData.cellColorVSImgColor:  cellData.cellAvgColor / cellData.imgAvgColor.
%        cellData.Hist:  Histogram of the object subimage.     
%        cellData.Class: Class of the cell. ([ ]=Discarded, [2] = Diffuse Aggregation, [3] = Translocated Aggregation).   
                        
% imageNames: Cell array containing the names of the images.                      

    files = dir([folder imageNameFilter]);
    numImages = size(files,1);
    

    cellLocationsByImage = cell(numImages,1);
    cellDataByImage = cell(numImages,1);
    imageNames = cell(numImages,1);

    for indImage=1:numImages,
        imageNames{indImage} = [folder files(indImage).name];
        disp(['Procesando imagen ',imageNames{indImage}]);
        [cellLocationsByImage{indImage} cellDataByImage{indImage}] = processImage(imageNames{indImage}, params);
    end
    
end

