% 
% Copyright (C) 2014 Luis de la Ossa, Jesús Martínez, Sara Sáez-Atiénzar, Joaquín Jordán.
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%  
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%  
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
%

function [objClassifier cellClassifier] = learnNaiveBayesModels(dataset)
% Learns two naive bayes classifiers from the data.

% dataset: Plain data. Contains two fields.

        % dataset.Class:  Class of the cell (numCelulas x 1).
                       % [ ]=Discarded, [2] = Diffuse Aggregation, [3] = Translocated Aggregation
                       
        % dataset.Input: Information related to each cell (numCelulas x 272).
%            1-->  Number of pixels of the cell.
%            2-->  Number of pixels over params.upperBackgroundThreshold.
%            3-->  Proportion of pixels over params.upperBackgroundThreshold.
%            4-->  Number of pixels over params.aggregationThreshold.
%            5-->  Proportion of pixels over params.aggregationThreshold.            
%            6-->  Lenght of the major axis of the ellipse containing the object.
%            7-->  Lenght of the minor axis of the ellipse containing the object.
%            8-->  Average color of the object
%            9-->  Number of pixels over params.upperBackgroundThreshold in the whole image.
%           10-->  Proportion  of pixels over params.upperBackgroundThreshold in the whole image.
%           11-->  Number of pixels over params.aggregationThreshold in the whole image.
%           12-->  Proportion  of pixels over params.aggregationThreshold in the whole image.                    
%           13-->  Average color of the image
%           14-->  cellData.propPixelsOverUpperThr / cellData.imgPropPixelsOverUpperThr.  
%           15-->  cellData.propPixelsOverAggregationThr / cellData.imgPropPixelsOverAggreThr.  
%           16-->  cellData.cellAvgColor / cellData.imgAvgColor.
%       17-272-->  Histogram of the object subimage.  


% objClassifier: Classifies objects between discarded (0) or transfected cells (1)
               % objClassifier.Variables: List of variables actually used.
               % objClassifier.Model: Naive Bayes model.

               
% cellClassifier: Classifies cells between diffuse aggregation (2) and translocated aggregation (3)
               % cellClassifier.Variables: List of variables actually used.
               % cellClassifier.Model: Naive Bayes model.

% Important

% The original dataset uses three classes:

%       0: Discarded
%       2: Diffuse aggregation
%       3: Translocated aggregatio.

% The first model distinguishes between discarded cells and the rest.
% Therefore, it makes a new assignation of classes. Discarded objects keep
% on having class 0. The rest  has class 1.

% The second model only considers objects with classes 2 and 3.

        
    %% First classifier. Determines which objects must be discarded.
    indDiscardedObjects = dataset.Class == 0;
    indAcceptedObjects =  dataset.Class > 0;
    
    variancesDiscardedObjects = var(dataset.Input(indDiscardedObjects,1:16));
    variancesAcceptedObjects = var(dataset.Input(indAcceptedObjects,1:16));
    
    validVariablesForDiscardedObjects = find(variancesDiscardedObjects>0);
    validVariablesForAcceptedObjects = find(variancesAcceptedObjects>0);
    objClassifier.Variables = intersect(validVariablesForDiscardedObjects,validVariablesForAcceptedObjects);    
    
    inputObjCl = dataset.Input(:,objClassifier.Variables);
    classObjCl = dataset.Class;
    classObjCl (indAcceptedObjects) = 1;
    
    objClassifier.Model = NaiveBayes.fit(inputObjCl,classObjCl);
    
    
    %% Second classifier. Determines the kind of transfected cell.
    indFuzzExamples = dataset.Class == 2;
    indTransExamples = dataset.Class == 3;

    variancesFuzzExamples = var(dataset.Input(indFuzzExamples,:));
    variancesTransExamples = var(dataset.Input(indTransExamples,:));

    validVariablesForFuzzExamples = find(variancesFuzzExamples>0);
    validVariablesForTransExamples = find(variancesTransExamples>0);
    cellClassifier.Variables = intersect(validVariablesForTransExamples,validVariablesForFuzzExamples);

    inputCellCl = dataset.Input(indAcceptedObjects,cellClassifier.Variables);
    classCellCl = dataset.Class(indAcceptedObjects);
    cellClassifier.Model = NaiveBayes.fit(inputCellCl,classCellCl);
end

