% 
% Copyright (C) 2014 Luis de la Ossa, Jesús Martínez, Sara Sáez-Atiénzar, Joaquín Jordán.
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%  
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%  
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
%

function [predictions confusionMatrices] = predict(dataset, objClassifier, cellClassifier)
% Predicts the output for the objects in the dataset given the classifiers.

% It is important to consider the way classes are managed.

% The first model determines which objects are discarded (class 0) and
% which are transfected cells (class 1). In order to elaborate the
% confusion matrix confusionMatrices.discarded, it is necessary to recode classes. Thus, objects with
% class 2 and 3 are considered objects with class 1.

% dataset: Plain data. Contains two fields.

        % dataset.Class:  Class of the cell (numCelulas x 1).
                       % [ ]=Discarded, [2] = Diffuse Aggregation, [3] = Translocated Aggregation
                       
        % dataset.Input: Information related to each cell (numCelulas x 272).
%            1-->  Number of pixels of the cell.
%            2-->  Number of pixels over params.upperBackgroundThreshold.
%            3-->  Proportion of pixels over params.upperBackgroundThreshold.
%            4-->  Number of pixels over params.aggregationThreshold.
%            5-->  Proportion of pixels over params.aggregationThreshold.            
%            6-->  Lenght of the major axis of the ellipse containing the object.
%            7-->  Lenght of the minor axis of the ellipse containing the object.
%            8-->  Average color of the object
%            9-->  Number of pixels over params.upperBackgroundThreshold in the whole image.
%           10-->  Proportion  of pixels over params.upperBackgroundThreshold in the whole image.
%           11-->  Number of pixels over params.aggregationThreshold in the whole image.
%           12-->  Proportion  of pixels over params.aggregationThreshold in the whole image.                    
%           13-->  Average color of the image
%           14-->  cellData.propPixelsOverUpperThr / cellData.imgPropPixelsOverUpperThr.  
%           15-->  cellData.propPixelsOverAggregationThr / cellData.imgPropPixelsOverAggreThr.  
%           16-->  cellData.cellAvgColor / cellData.imgAvgColor.
%       17-272-->  Histogram of the object subimage.  



% objClassifier: Classifies objects between discarded (0) or transfected cells (1)
               % objClassifier.Variables: List of variables actually used.
               % objClassifier.Model: Naive Bayes model.

               
% cellClassifier: Classifies cells between diffuse aggregation (2) and translocated aggregation (3)
               % cellClassifier.Variables: List of variables actually used.
               % cellClassifier.Model: Naive Bayes model.


% predictions: Contains the predictions for each object.
             % predictions.discarded: Contains predictions (0 or 1) made by objClassifier.  
             % predictions.cellType:  Contains the predictions (2 or 3) made by cellClassifier. 

% confusionMatrices: 
                   % confusionMatrices.discarded: Confusion matrix produced by objClassifier
                   % confusionMatrices.cellType: Confusion matrix produced by cellClassifier

    %% First stage. The model detects which objects must be discarded.
    predictions.discarded = objClassifier.Model.predict(dataset.Input(:,objClassifier.Variables)); 
    objClasses = dataset.Class; 
    indCells = objClasses > 0; 
    objClasses(indCells)=1; 
    confusionMatrices.discarded = confusionmat(objClasses, predictions.discarded);
    
    %% Second stage. Cells are classified.
    indCells = predictions.discarded > 0;
    predictions.cellType = cellClassifier.Model.predict(dataset.Input(indCells,cellClassifier.Variables));
    cellClasses = dataset.Class(indCells);
    confusionMatrices.cellType = confusionmat(cellClasses, predictions.cellType);    
end

