% 
% Copyright (C) 2014 Luis de la Ossa, Jesús Martínez, Sara Sáez-Atiénzar, Joaquín Jordán.
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%  
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%  
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
%

function [cellLocations cellData] = processImage(image,params)
% Extracts the data relative to the cells in the image.

% image: Name of the file containing the image.

% params: Structure containing the parameters of the algorithm. 
%
%      params.lowerBackgroundThreshold: Pixels over this value are considered interest objects.
%      params.upperBackgroundThreshold: This threshold is used to segment accumulations of cells.
%      params.aggregationThreshold: Regions above this threshold are considered as aggregations of proteins.
%      params.minCellSize: Minimum size of a cell.
%      params.minCumSize: Minimum size of an accumulation of cells.
%      params.smallCellMaxSize: Minimum size of a cell considered as ``small''
%      params.askForClass: Whether to ask the user for kind of object or not (usefull for automatic quantification).
           

% cellLocations: Array of structos containing the position of each object in the image.
%             cellLocations.BoundingBox: Coordinates of the rectangle containing the object (4x1)
%             cellLocations.Centroid: Centroid of the object.   

% cellData: Array of structs with the information relative to each object.
%        cellData.numPixels: Number of pixels of the cell.
%        cellData.numPixelsOverUpperThr:  Number of pixels over params.upperBackgroundThreshold.
%        cellData.propPixelsOverUpperThr:  Proportion of pixels over params.upperBackgroundThreshold.
%        cellData.numPixelsOverAggregationThr:  Number of pixels over params.aggregationThreshold.
%        cellData.propPixelsOverAggregationThr:  Proportion of pixels over params.aggregationThreshold.         
%        cellData.majorAxisLength:  Lenght of the major axis of the ellipse containing the object.
%        cellData.minorAxisLength:  Lenght of the minor axis of the ellipse containing the object.
%        cellData.cellAvgColor:  Average color of the object
%        cellData.imgNumPixelsOverUpperThr:  Number of pixels over params.upperBackgroundThreshold in the whole image.
%        cellData.imgPropPixelsOverUpperThr: Proportion  of pixels over params.upperBackgroundThreshold in the whole image.
%        cellData.imgNumPixelsOverAggreThr:  Number of pixels over params.aggregationThreshold in the whole image.
%        cellData.imgPropPixelsOverAggreThr: Proportion  of pixels over params.aggregationThreshold in the whole image.                    
%        cellData.imgAvgColor:  Average color of the image.
%        cellData.cellPOUTvsimgPOUT:  cellData.propPixelsOverUpperThr / cellData.imgPropPixelsOverUpperThr.
%        cellData.cellPOATvsimgPOAT:  cellData.propPixelsOverAggregationThr / cellData.imgPropPixelsOverAggreThr.
%        cellData.cellColorVSImgColor:  cellData.cellAvgColor / cellData.imgAvgColor.
%        cellData.Hist:  Histogram of the object subimage.     
%        cellData.Class: Kind of the cell. ([ ]=Discarded, [2] = Diffuse Aggregation, [3] = Translocated Aggregation).   



    %% Data structs which store the information of the cells.
    % Locations
    cellLocations = struct('BoundingBox',zeros(4,1),'Centroid',zeros(2,1));
    % Data 
    cellData = struct( 'numPixels',0, ... 
                       'numPixelsOverUpperThr',0,... 
                       'propPixelsOverUpperThr',0, ... 
                       'numPixelsOverAggregationThr',0,... 
                       'propPixelsOverAggregationThr',0, ...          
                       'majorAxisLength',0, ...  
                       'minorAxisLength',0, ...  
                       'cellAvgColor',0, ... 
                       'imgNumPixelsOverUpperThr',0,... 
                       'imgPropPixelsOverUpperThr',0,... 
                       'imgNumPixelsOverAggreThr',0,... 
                       'imgPropPixelsOverAggreThr',0,...                     
                       'imgAvgColor',0,... 
                       'cellPOUTvsimgPOUT',0,...  
                       'cellPOATvsimgPOAT',0,...   
                       'cellColorVSImgColor',0,...
                       'Hist',zeros(256,1),...    
                       'Class',0); 
 
                   
    %% Reads the image and converts it to gray scale. 
    imageRGB = imread(image);
    imageGray = rgb2gray(imageRGB);
    
    %% Extracts relevant data from each image.
    imgNumPixels = numel(imageGray); 
    imgAvgColor = mean(imageGray(:)); 
    imgNumPixelsOverUpperThr = numel(find(imageGray>params.upperBackgroundThreshold)); 
    imgPropPixelsOverUpperThr = imgNumPixelsOverUpperThr / imgNumPixels; 
    imgNumPixelsOverAggregationThr = numel(find(imageGray>params.aggregationThreshold)); 
    imgPropPixelsOverAggregationThr = imgNumPixelsOverAggregationThr / imgNumPixels; 
    
    
    %% First selection of objects (color>params.lowerBackgroundThreshold)
    element = strel('disk',7);
    imageDilated = imdilate(imageGray,element);
    imageGrayDilated = imfill(imageDilated,'holes');
    % figure, imshow(imageGrayDilated,[]), title('Gray scale image (dilated)');
    imageBW = im2bw(imageGrayDilated, params.lowerBackgroundThreshold/255);
    imageBWClean = bwareaopen(imageBW,params.minCellSize);
    % figure, imshow(imageBWClean,[]), title('Mask with no noise');
    
    
    %% This fragment discards objects which are both small and dark
    objects = bwconncomp(imageBWClean, 8);
    objectFrames = regionprops(objects,'Area','BoundingBox','Centroid','MajorAxisLength','MinorAxisLength','PixelList');
    selected = false(objects.NumObjects); 
    for indObject=1:objects.NumObjects,
         if numel(objects.PixelIdxList{indObject}) > params.smallCellMaxSize 
             selected(indObject) = true; 
             continue;
         end
         pixelsObject = imageGray(objects.PixelIdxList{indObject});
         aggregatedPixels = find(pixelsObject>=params.aggregationThreshold);
         if numel(aggregatedPixels)>params.minCellSize/2,
             selected(indObject) = true;
         end
    end
    selectedObjects = find(selected==true);
    objects.PixelIdxList = objects.PixelIdxList(selectedObjects);
    objects.NumObjects = numel(selectedObjects);
    objectFrames = objectFrames(selectedObjects);
    
    
    
    %% Each one of the detected objects is processed individually, as some of them can be accumulation of cells.
    currentCellId = 0;    
    for indObject=1:objects.NumObjects,             
        objectBoundingBox =  objectFrames(indObject).BoundingBox'; 
        objectCentroid = objectFrames(indObject).Centroid';        
        pixelsObject = objects.PixelIdxList{indObject};
        maskObject = zeros(size(imageGray),'uint8');
        maskObject(pixelsObject) = 1;
        imageObject= imageGray.*maskObject;      
        
        %% Applies a second thresholdind to determine the number of objects in the image
        element = strel('disk',2);
        imageObjectDilated = imdilate(imageObject,element);
        imageObjectDilated = imfill(imageObjectDilated,'holes');
        % figure, imshow(imageObjectDilated,[]), title('Imagen del objeto dilatada');        
        imageObjectBW = im2bw(imageObjectDilated, params.upperBackgroundThreshold/255);
        imageObjectBWClean = bwareaopen(imageObjectBW,params.smallCellMaxSize);
        % figure, imshow(imageObjectBWClean,[]), title('Objeto procesado');                
        cells = bwconncomp(imageObjectBWClean, 8);
        cellFrames = regionprops(cells,'Area','BoundingBox','Centroid','MajorAxisLength','MinorAxisLength','PixelList');        
        if (cells.NumObjects<2) || numel(pixelsObject)<params.minCumSize,
            numCellsInObject = 1; 
        else
            numCellsInObject = 2; 
        end
        
        
        %% In case the object is a cell, data are directly stored
        if numCellsInObject==1,        
            currentCellId = currentCellId + 1;
            cellLocations(currentCellId).BoundingBox = objectBoundingBox;
            cellLocations(currentCellId).Centroid = objectCentroid;
            
            cellData(currentCellId).numPixels = numel(pixelsObject);
            cellData(currentCellId).numPixelsOverUpperThr = numel(find(imageObject>params.upperBackgroundThreshold));
            cellData(currentCellId).propPixelsOverUpperThr = cellData(currentCellId).numPixelsOverUpperThr / cellData(currentCellId).numPixels; 
            cellData(currentCellId).numPixelsOverAggregationThr = numel(find(imageObject>params.aggregationThreshold));
            cellData(currentCellId).propPixelsOverAggregationThr = cellData(currentCellId).numPixelsOverAggregationThr / cellData(currentCellId).numPixels;
            cellData(currentCellId).majorAxisLength = objectFrames(indObject).MajorAxisLength;
            cellData(currentCellId).minorAxisLength = objectFrames(indObject).MinorAxisLength;                 
            cellData(currentCellId).cellAvgColor = mean(imageGray(pixelsObject)); 
            cellData(currentCellId).imgNumPixelsOverUpperThr = imgNumPixelsOverUpperThr;
            cellData(currentCellId).imgPropPixelsOverUpperThr = imgPropPixelsOverUpperThr;
            cellData(currentCellId).imgNumPixelsOverAggreThr = imgNumPixelsOverAggregationThr;
            cellData(currentCellId).imgPropPixelsOverAggreThr = imgPropPixelsOverAggregationThr;
            cellData(currentCellId).imgAvgColor = imgAvgColor;
            cellData(currentCellId).cellPOUTvsimgPOUT = cellData(currentCellId).propPixelsOverUpperThr / cellData(currentCellId).imgPropPixelsOverUpperThr;
            cellData(currentCellId).cellPOATvsimgPOAT = cellData(currentCellId).propPixelsOverAggregationThr / cellData(currentCellId).imgPropPixelsOverAggreThr;
            cellData(currentCellId).cellColorVSImgColor = cellData(currentCellId).cellAvgColor/cellData(currentCellId).imgAvgColor;        
         
            
            objectSubImage = imcrop(imageObject,objectBoundingBox);
            % figure, imshow(objectSubImage,[]), title('Objeto');                
            hist = imhist(objectSubImage);
            hist = hist / sum(hist);
            cellData(currentCellId).Hist = hist;

   
            if (params.askForClass)
                imagesc(imageRGB);
                rectangle('Position',objectBoundingBox, 'LineWidth',2, 'EdgeColor','w');
                clase = input('Which class is the object ? \n   [ ]: Discarded \n   [2]: Cell (diffuse) \n   [3]: Cell (translocated) \n');
                if (clase==2)
                    cellData(currentCellId).Class = 2;
                elseif (clase==3)
                    cellData(currentCellId).Class = 3;
                else
                    cellData(currentCellId).Class = 0;
                end;
            else
                cellData(currentCellId).Class = 0;
            end   

            continue; 
        end 
    
    
        %% This part of the code is only executed if there is an accumulation of cells (numCells=2)
        sizeCells = cellfun('length',cells.PixelIdxList);
        [~, indMaxSizes] = sort(sizeCells,'descend');    
        for numCellCum=1:2,
            currentCellId = currentCellId + 1; 
            idCellCum = indMaxSizes(numCellCum); 
            pixelsCell = cells.PixelIdxList{idCellCum}; 
            maskCell = zeros(size(imageGray),'uint8');
            maskCell(pixelsCell) = 1;     
            imageCell= imageGray.*maskCell;     
            cellBoundingBox =  cellFrames(idCellCum).BoundingBox'; 
            cellCentroid = cellFrames(idCellCum).Centroid';         

            cellLocations(currentCellId).BoundingBox = cellBoundingBox;
            cellLocations(currentCellId).Centroid = cellCentroid;

            cellData(currentCellId).numPixels = numel(pixelsCell);
            cellData(currentCellId).numPixelsOverUpperThr = numel(find(imageCell>params.upperBackgroundThreshold));
            cellData(currentCellId).propPixelsOverUpperThr = cellData(currentCellId).numPixelsOverUpperThr / cellData(currentCellId).numPixels; 
            cellData(currentCellId).numPixelsOverAggregationThr = numel(find(imageCell>params.aggregationThreshold));
            cellData(currentCellId).propPixelsOverAggregationThr = cellData(currentCellId).numPixelsOverAggregationThr / cellData(currentCellId).numPixels;
            cellData(currentCellId).majorAxisLength = objectFrames(idCellCum).MajorAxisLength;
            cellData(currentCellId).minorAxisLength = objectFrames(idCellCum).MinorAxisLength;                 
            cellData(currentCellId).cellAvgColor = mean(imageGray(pixelsCell)); % Para que descarte el fondo se hace directamente
            cellData(currentCellId).imgNumPixelsOverUpperThr = imgNumPixelsOverUpperThr;
            cellData(currentCellId).imgPropPixelsOverUpperThr = imgPropPixelsOverUpperThr;
            cellData(currentCellId).imgNumPixelsOverAggreThr = imgNumPixelsOverAggregationThr;
            cellData(currentCellId).imgPropPixelsOverAggreThr = imgPropPixelsOverAggregationThr;
            cellData(currentCellId).imgAvgColor = imgAvgColor;
            cellData(currentCellId).cellPOUTvsimgPOUT = cellData(currentCellId).propPixelsOverUpperThr / cellData(currentCellId).imgPropPixelsOverUpperThr;
            cellData(currentCellId).cellPOATvsimgPOAT = cellData(currentCellId).propPixelsOverAggregationThr / cellData(currentCellId).imgPropPixelsOverAggreThr;
            cellData(currentCellId).cellColorVSImgColor = cellData(currentCellId).cellAvgColor/cellData(currentCellId).imgAvgColor;        
                   

            cellSubImage = imcrop(imageCell,cellBoundingBox);
            % figure, imshow(objectSubImage,[]), title('Objeto');                
            hist = imhist(cellSubImage);
            hist = hist / sum(hist);
            cellData(currentCellId).Hist = hist;  

            if (params.askForClass)
                imagesc(imageRGB);
                rectangle('Position',cellBoundingBox, 'LineWidth',2, 'EdgeColor','w');
                clase = input('Which class is the object ? \n   [ ]: Discarded \n   [2]: Cell (diffuse) \n   [3]: Cell (translocated) \n');                if (clase==2)
                    cellData(currentCellId).Class = 2;
                elseif (clase==3)
                    cellData(currentCellId).Class = 3;
                else
                    cellData(currentCellId).Class = 0;
                end;
            else
                cellData(currentCellId).Class = 0;                
            end                 
        end 
    end 
end 

